/* $Id: ApmMeter.c,v 1.2 1999/11/06 17:10:56 leonard Exp $ */
/* (c) 1999, David Leonard */
/*
 * To-do:
 *   add a Sleep() action that puts the machine into sleep mode ('<Key>z'?)
 *   watch the Visibility and don't poll if not visible
 *   or add resources to specify a shell commands to run on entry/exit of a zone
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include <X11/Xlib.h>
#include <X11/IntrinsicP.h>
#include <X11/StringDefs.h>

#include "ApmMeterP.h"

#if 1
#define dprintf(fmt , args...)	 /* nil */
#else
#define dprintf(fmt , args...) \
	fprintf(stderr, fmt , ## args)
#endif

static XtResource resources[] = {
	/* ChargingForeground chargingForeground */
	{
		XtNchargingForeground, XtCChargingForeground,
		XtRPixel, sizeof(Pixel),
		XtOffsetOf(ApmMeterRec, apmmeter.charging_foreground),
		XtRString, "green"
	},
	/* ChargingForeground chargingBackground */
	{
		XtNchargingBackground, XtCChargingBackground,
		XtRPixel, sizeof(Pixel),
		XtOffsetOf(ApmMeterRec, apmmeter.charging_background),
		XtRString, "white"
	},
	/* HighForeground highForeground */
	{
		XtNhighForeground, XtCHighForeground,
		XtRPixel, sizeof(Pixel),
		XtOffsetOf(ApmMeterRec, apmmeter.high_foreground),
		XtRString, "black"
	},
	/* HighForeground highBackground */
	{
		XtNhighBackground, XtCHighBackground,
		XtRPixel, sizeof(Pixel),
		XtOffsetOf(ApmMeterRec, apmmeter.high_background),
		XtRString, "white"
	},
	/* LowForeground lowForeground */
	{
		XtNlowForeground, XtCLowForeground,
		XtRPixel, sizeof(Pixel),
		XtOffsetOf(ApmMeterRec, apmmeter.low_foreground),
		XtRString, "red"
	},
	/* LowForeground lowBackground */
	{
		XtNlowBackground, XtCLowBackground,
		XtRPixel, sizeof(Pixel),
		XtOffsetOf(ApmMeterRec, apmmeter.low_background),
		XtRString, "white"
	},
	/* DangerousForeground dangerousForeground */
	{
		XtNdangerousForeground, XtCDangerousForeground,
		XtRPixel, sizeof(Pixel),
		XtOffsetOf(ApmMeterRec, apmmeter.dangerous_foreground),
		XtRString, "red"
	},
	/* DangerousForeground dangerousBackground */
	{
		XtNdangerousBackground, XtCDangerousBackground,
		XtRPixel, sizeof(Pixel),
		XtOffsetOf(ApmMeterRec, apmmeter.dangerous_background),
		XtRString, "yellow"
	},
	/* CriticalForeground criticalForeground */
	{
		XtNcriticalForeground, XtCCriticalForeground,
		XtRPixel, sizeof(Pixel),
		XtOffsetOf(ApmMeterRec, apmmeter.critical_foreground),
		XtRString, "red"
	},
	/* CriticalForeground criticalBackground */
	{
		XtNcriticalBackground, XtCCriticalBackground,
		XtRPixel, sizeof(Pixel),
		XtOffsetOf(ApmMeterRec, apmmeter.critical_background),
		XtRString, "yellow"
	},
	/* HighLevel highLevel */
	{
		XtNhighLevel, XtCHighLevel,
		XtRFloat, sizeof(float),
		XtOffsetOf(ApmMeterRec, apmmeter.high_level),
		XtRString, "0.50"
	},
	/* LowLevel lowLevel */
	{
		XtNlowLevel, XtCLowLevel,
		XtRFloat, sizeof(float),
		XtOffsetOf(ApmMeterRec, apmmeter.low_level),
		XtRString, "0.20"
	},
	/* DangerousLevel dangerousLevel */
	{
		XtNdangerousLevel, XtCDangerousLevel,
		XtRFloat, sizeof(float),
		XtOffsetOf(ApmMeterRec, apmmeter.dangerous_level),
		XtRString, "0.10"
	},
	/* Update update */
	{
		XtNupdate, XtCUpdate,
		XtRDimension, sizeof(Dimension),
		XtOffsetOf(ApmMeterRec, apmmeter.update),
		XtRImmediate, (XtPointer)5000
	}
};

static void Initialize(Widget, Widget, ArgList, Cardinal *);
static void Destroy(Widget);
static Boolean SetValues(Widget, Widget, Widget, ArgList, Cardinal *);

static void timeout_proc(XtPointer, XtIntervalId *);
static void amUpdate(ApmMeterWidget);
static void amSample(ApmMeterWidget);

ApmMeterClassRec apmMeterClassRec = {
	/* core_class fields */
	{
		(WidgetClass) &scrollbarClassRec, /* superclass */
		"ApmMeter",			/* class_name */
		sizeof(ApmMeterRec),		/* widget_size */
		NULL,				/* class_initialize */
		NULL,				/* class_part_initialize */
		False,				/* class_inited */
		Initialize,			/* initialize */
		NULL,				/* initialize_hook */
		XtInheritRealize,		/* realize */
		NULL,				/* actions */
		0,				/* num_actions */
		resources,			/* resources */
		XtNumber(resources),		/* num_resources */
		NULLQUARK,			/* xrm_class */
		True,				/* compress_motion */
		True,				/* compress_exposure */
		True,				/* compress_enterleave */
		False,				/* visible_interest */
		Destroy,			/* destroy */
		XtInheritResize,		/* resize */
		XtInheritExpose,		/* expose */
		NULL,				/* set_values */
		NULL,				/* set_values_hook */
		XtInheritSetValuesAlmost,	/* set_values_almost */
		NULL,				/* get_values_hook */
		XtInheritAcceptFocus,		/* accept_focus */
		XtVersion,			/* version */
		NULL,				/* callback_private */
		NULL,				/* tm_table */
		XtInheritQueryGeometry,		/* query_geometry */
		XtInheritDisplayAccelerator,	/* display_accelerator */
		NULL				/* extension */
	},
	{ /* simple fields */
		XtInheritChangeSensitive	/* change_sensitive */
	},
	{ /* scrollbar fields */
		0				/* ignore */
	},
	{ /* apmMeter fields */
		0				/* empty */
	}
};

WidgetClass apmMeterWidgetClass = (WidgetClass) &apmMeterClassRec;

static void
timeout_proc(client_data, timer)
	XtPointer	client_data;
	XtIntervalId *	timer;
{
	ApmMeterWidget	aw = (ApmMeterWidget)client_data;

	dprintf("timeout_proc\n");
	amSample(aw);
	amUpdate(aw);
	aw->apmmeter.timer = XtAppAddTimeOut(
	    XtWidgetToApplicationContext((Widget)aw),
	    aw->apmmeter.update, timeout_proc, (XtPointer)aw);
}

static void
Initialize(request, new, args, num_args)
	Widget		request;
	Widget		new;
	ArgList		args;
	Cardinal *	num_args;
{
	ApmMeterWidget	aw = (ApmMeterWidget)new;

	dprintf("Initialize\n");

	/* Start update cycle. */
	timeout_proc((XtPointer)aw, NULL);
}

static void
Destroy(widget)
	Widget		widget;
{
	ApmMeterWidget	aw = (ApmMeterWidget)widget;

	dprintf("Destroy\n");
	/* Stop update cycle. */
	XtRemoveTimeOut(aw->apmmeter.timer);
}

static void
amSample(aw)
	ApmMeterWidget aw;
{
	int	battery;
	int	adaptor;

	dprintf("Sample\n");
	getcharge(&battery, &adaptor);		/* XXX what do do when fails? */

	aw->apmmeter.charging = adaptor ? True : False;
	aw->apmmeter.charge = (battery / 100.0);
}

static void
amUpdate(aw)
	ApmMeterWidget aw;
{
	Pixel fg, bg;

	dprintf("Update\n");

	dprintf("charge = %f (dang=%f low=%f high=%f)\n",
		aw->apmmeter.charge,
		aw->apmmeter.dangerous_level,
		aw->apmmeter.low_level,
		aw->apmmeter.high_level);
	dprintf("charg=(%d,%d) crit=(%d,%d) dang=(%d,%d), low=(%d,%d), "
			"hi=(%d,%d)\n",
		aw->apmmeter.charging_foreground,
		aw->apmmeter.charging_background,
		aw->apmmeter.critical_foreground,
		aw->apmmeter.critical_background,
		aw->apmmeter.dangerous_foreground,
		aw->apmmeter.dangerous_background,
		aw->apmmeter.low_foreground,
		aw->apmmeter.low_background,
		aw->apmmeter.high_foreground,
		aw->apmmeter.high_background);

	/* Decide on the new colours.. */
	if (aw->apmmeter.charging) {
		fg = aw->apmmeter.charging_foreground;
		bg = aw->apmmeter.charging_background;
	} else if (aw->apmmeter.charge < aw->apmmeter.dangerous_level) {
		fg = aw->apmmeter.critical_foreground;
		bg = aw->apmmeter.critical_background;
	} else if (aw->apmmeter.charge < aw->apmmeter.low_level) {
		fg = aw->apmmeter.dangerous_foreground;
		bg = aw->apmmeter.dangerous_background;
	} else if (aw->apmmeter.charge < aw->apmmeter.high_level) {
		fg = aw->apmmeter.low_foreground;
		bg = aw->apmmeter.low_background;
	} else {
		fg = aw->apmmeter.high_foreground;
		bg = aw->apmmeter.high_background;
	}
	XawScrollbarSetThumb((Widget)aw, 0.0, aw->apmmeter.charge);
	if (XtIsRealized((Widget)aw)) { /* Why??!!!! */
		XtVaSetValues((Widget)aw, 
			XtNforeground, fg,
			XtNbackground, bg,
			NULL);
	}
}

static Boolean
SetValues(current, request, new, args, num_args)
	Widget	current, request, new;
	ArgList	args;
	Cardinal *num_args;
{
	ApmMeterWidget	aw = (ApmMeterWidget)new;
	ApmMeterPart	*before = &((ApmMeterWidget)current)->apmmeter;
	ApmMeterPart	*after = &((ApmMeterWidget)request)->apmmeter;
	int redraw = False;

	dprintf("SetValues\n");

	if (XtIsRealized(new) &&
	   ((before->charging_foreground  != after->charging_foreground)
	||  (before->charging_background  != after->charging_background)
	||  (before->high_foreground      != after->high_foreground)
	||  (before->high_background      != after->high_background)
	||  (before->low_foreground       != after->low_foreground)
	||  (before->low_background       != after->low_background)
	||  (before->dangerous_foreground != after->dangerous_foreground)
	||  (before->dangerous_background != after->dangerous_background)
	||  (before->critical_foreground  != after->critical_foreground)
	||  (before->critical_background  != after->critical_background)
	||  (before->high_level           != after->high_level)
	||  (before->low_level            != after->low_level)
	||  (before->dangerous_level      != after->dangerous_level))) {
		amUpdate(aw);
		redraw = True;
	}
	/*
	redraw = (*scrollbarClassRec.core_class.set_values)(current, 
	    request, new, args, num_args) || redraw;
	*/
	return redraw;
}
