/*	$OpenBSD: apm.c,v 1.3 1998/10/29 18:21:44 mickey Exp $	*/

/*
 *  Copyright (c) 1996 John T. Kohl
 *  All rights reserved.
 * 
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR `AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * 
 */

#include <stdio.h>
#include <errno.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <err.h>
#include <string.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <sys/ioctl.h>
#include <machine/apmvar.h>
#include "pathnames.h"
#include "apm-proto.h"

int do_zzz(const char *pn, enum apm_action action);
int open_socket(const char *pn);
int send_command(int fd,
		 struct apm_command *cmd,
		 struct apm_reply *reply);

int
send_command(int fd,
	     struct apm_command *cmd,
	     struct apm_reply *reply)
{
    /* send a command to the apm daemon */
    cmd->vno = APMD_VNO;

    if (send(fd, cmd, sizeof(*cmd), 0) == sizeof(*cmd)) {
	if (recv(fd, reply, sizeof(*reply), 0) != sizeof(*reply)) {
	    warn("invalid reply from APM daemon");
	    return 1;
	}
    } else {
	warn("invalid send to APM daemon");
	return 1;
    }
    return 0;
}

int
do_zzz(const char *pn, enum apm_action action)
{
    struct apm_command command;
    struct apm_reply reply;
    int fd;

    switch (action) {
    case SUSPEND:
	command.action = SUSPEND;
	break;
    case STANDBY:
	command.action = STANDBY;
	break;
    default:
	return -1;
    }
    fd = open_socket(pn);
    if (fd == -1)
	warn("cannot open connection to APM daemon");
    printf("Suspending system...\n");
    return send_command(fd, &command, &reply);
}

int
open_socket(const char *sockname)
{
    int sock, errr;
    struct sockaddr_un s_un;

    sock = socket(AF_UNIX, SOCK_STREAM, 0);
    if (sock == -1) {
	warn("cannot create local socket");
	return -1;
    }

    s_un.sun_family = AF_UNIX;
    strncpy(s_un.sun_path, sockname, sizeof(s_un.sun_path));
    s_un.sun_len = SUN_LEN(&s_un);
    if (connect(sock, (struct sockaddr *)&s_un, s_un.sun_len) == -1) {
	errr = errno;
	close(sock);
	errno = errr;
	warn("apmd(8) not running? connect");
	return -1;
    }
    return sock;
}


int
getcharge(battery, adaptor)
	int	*battery;		/* battery life as percentage */
	int	*adaptor;		/* adaptor state  */
{
	int fd;
	enum apm_action action = NONE;
	struct apm_command command;
	struct apm_reply reply;

	fd = open_socket(_PATH_APM_SOCKET);
	if (fd == -1) {
		/* open the device directly and get status (old code) */
		fd = open(_PATH_APM_NORMAL, O_RDONLY);
		if (fd == -1) {
			warn("%s", _PATH_APM_NORMAL);
			return -1;
		}
	        if (ioctl(fd, APM_IOC_GETPOWER, &reply.batterystate) == -1) {
			warn("APM_IOC_GETPOWER");
			close(fd);
			return -1;
		}
		close(fd);
	} else {
		if (send_command(fd, &command, &reply) != 0) {
			warn("send_command");
			close(fd);
			return -1;
		}
		close(fd);
	}

	*battery = reply.batterystate.battery_life;
	switch (reply.batterystate.ac_state) {
	case APM_AC_ON:		*adaptor = 1; break;
	default:		*adaptor = 0; break;
	}

	return (0);
}
